<?php

namespace App\Http\Middleware;

use App\Http\Controllers\Api\LoginController;
use App\Service\EmployeeService;
use Closure;
use App\Service\TokenService;

class CheckLogin
{
    /**
     * Handle an incoming request.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \Closure  $next
     * @return mixed
     */
    public function handle($request, Closure $next)
    {
        $token=$request->header('Authorization');
        if (!isset($token)){
            return  response()->json(['code'=>1,'msg'=>'缺少token','data'=>null]);
        }

        //登录来源前缀清除
        foreach (LoginController::$port as $key => $value){
            if (strpos($token,$key) !== false) {
                $token = str_replace($key, $value, $token);
                break;
            }
        }

        //校验token
        $result = TokenService::verifyToken($token);
        if ($result < 0){
            return response()->json(['code'=>1,'msg'=>TokenService::error[$result],'data'=>null]);
        }

        //校验用户
        $checkResult = EmployeeService::checkUser($result);
        list($state, $data) = $checkResult;
        if(! $state) return response()->json(['code'=>1,'msg'=>$data,'data'=>null]);

        //人员角色
        $data['role'] = EmployeeService::getPersonRole($result);
        //部门权限
        $data['rule_depart'] = EmployeeService::getPersonDepart($result);
        //写入user信息
        $request->userData = $data;

        return $next($request);
    }
}
